const std = @import("std");
const sdl = @import("sdl2");
const gl = @import("zgl");

//const Game = @import("Game.zig");
//const MainMenu = @import("MainMenu.zig");

//const State = @import("flow.zig").State;

pub fn main() !void {
    try sdl.init(.{ .video = true, .audio = true, .events = true });
    defer sdl.quit();

    // Set OpenGL version
    try sdl.gl.setAttribute(.{ .context_major_version = 3 });
    try sdl.gl.setAttribute(.{ .context_minor_version = 3 });

    const window = try sdl.createWindow(
        "USG",
        .{ .centered = {} },
        .{ .centered = {} },
        1280,
        720,
        .{ .opengl = true, .shown = true }
    );
    defer window.destroy();

    const ctx = try sdl.gl.createContext(window);
    defer sdl.gl.deleteContext(ctx);

    const program = gl.Program.create();
    {
        const vs = gl.Shader.create(.vertex);
        defer vs.delete();
        vs.source(1, &.{@embedFile("shaders/vector.vs")});
        vs.compile();
        const fs = gl.Shader.create(.fragment);
        defer fs.delete();
        fs.source(1, &.{@embedFile("shaders/fragment.fs")});
        fs.compile();
        program.attach(vs);
        defer program.detach(vs);
        program.attach(fs);
        defer program.detach(fs);
        program.link();
    }
    program.use();

    var vertex_array = gl.VertexArray.gen();
    vertex_array.bind();

    const vertex_buffer = [_]f32{
        -1.0, -1.0, 0.0,
        1.0, -1.0, 0.0,
        0.0, 1.0, 0.0,
    };

    const buf = gl.Buffer.gen();
    buf.bind(.array_buffer);
    buf.data(f32, &vertex_buffer, .static_draw);

    //var renderer = SDL.SDL_CreateRenderer(window, -1, SDL.SDL_RENDERER_PRESENTVSYNC | SDL.SDL_RENDERER_ACCELERATED) orelse return;
    //defer _ = SDL.SDL_DestroyRenderer(renderer);

    //var main_menu = MainMenu.init();
    //var game = Game.init(renderer);

    //var current_state: State = main_menu.state;

    //_ = SDL.IMG_Init(SDL.IMG_INIT_JPG);

    gl.clearColor(0.91, 0.97, 1.00, 1.00);

    mainLoop: while (true) {
        while (sdl.pollEvent()) |ev| {
            switch (ev) {
                .quit => break :mainLoop,
                else => {},
            }
        }

        gl.clear(.{ .color = true });

        gl.enableVertexAttribArray(0);
        buf.bind(.array_buffer);

        gl.vertexAttribPointer(0, 3, .float, false, 0, 0);
        gl.drawArrays(.triangles, 0, 3);
        gl.disableVertexAttribArray(0);

        //current_state = switch (current_state) {
            //.main_menu => main_menu.tick(),
            //.game => game.tick(),
        //};

        sdl.gl.swapWindow(window);
    }
}
